import 'package:flutter/material.dart';

import '../data/api_response.dart';
import '../services/base_service.dart';
import '../models/models.dart';
import '../services/services.dart';

class ProductoViewModel<T> extends ChangeNotifier {
  String _busqueda = "";
  String get busqueda => _busqueda;

  List<Producto> _productos = [];
  Producto? _selectedModelo;
  bool _isLoading = false;
  int? _selectedCategoriaId = 0;

  List<Producto> get productos => _productos;
  Producto? get selectedProducto => _selectedModelo;
  bool get isLoading => _isLoading;

  int pagina = 1;
  int totalPaginas = 1;
  int limite = 10;

  setBusqueda(String value) {
    _busqueda = value;
    notifyListeners();
  }

  Future<void> fetchLocalAll() async {
    _selectedCategoriaId = 0;
    var db = await RepoService().db;
    var query = await db!.query(
      'Producto',
      orderBy: 'idLocal asc',
    );
    List<Producto> aux = [];
    for (var element in query) {
      Producto producto = Producto.fromJson(element);
      aux.add(producto);
    }
    _productos = aux;
    notifyListeners();
  }

  Future<void> fetchLocalByID({required int idCategoria}) async {
    if (idCategoria == _selectedCategoriaId) {
      _selectedCategoriaId = 0;
      fetchLocalAll();
      return;
    }

    //Se utiliza para actualizar la UI del botón
    _selectedCategoriaId = idCategoria;

    var db = await RepoService().db;
    var query = await db!.query(
      'Producto',
      where: 'idCategoria ==?',
      whereArgs: ['$idCategoria'],
      orderBy: 'idLocal asc',
    );
    List<Producto> aux = [];
    for (var element in query) {
      Producto producto = Producto.fromJson(element);
      aux.add(producto);
    }
    _productos = aux;
    notifyListeners();
  }

  Future<void> fetchLocalByName({required String nombre}) async {
    var db = await RepoService().db;
    var query = await db!.query(
      'Producto',
      where: 'nombre LIKE "%$nombre%"',
      orderBy: 'idLocal asc',
    );
    List<Producto> aux = [];
    for (var element in query) {
      Producto producto = Producto.fromJson(element);
      aux.add(producto);
    }
    _productos = aux;
    notifyListeners();
  }

  void setIsLoading(bool loading) {
    _isLoading = loading;
    notifyListeners();
  }
}
